package net.knarcraft.permissionsigns.command;

import net.knarcraft.knarlib.formatting.StringFormatter;
import net.knarcraft.permissionsigns.PermissionSigns;
import net.knarcraft.permissionsigns.config.Translatable;
import net.knarcraft.permissionsigns.container.PermissionSign;
import net.knarcraft.permissionsigns.manager.SignCreationManager;
import org.bukkit.Bukkit;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;

import java.util.List;

/**
 * The command used to create a new permission sign
 */
public class CreateCommand implements CommandExecutor {

    @Override
    public boolean onCommand(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label,
                             @NotNull String[] args) {
        // /ps create <name> <permission,permission> <cost> <duration> to create a new permission-sign
        //Name and permission(s) required, but duration and cost optional
        String usage = "/ps create <name> <description> <permission,permission> [cost] [duration] - Used for creating" +
                " a new permission sign";
        StringFormatter stringFormatter = PermissionSigns.getInstance().getStringFormatter();

        if (!(sender instanceof Player)) {
            stringFormatter.displayErrorMessage(sender, Translatable.COMMAND_PLAYER_ONLY);
            return false;
        }
        if (!sender.hasPermission("permissionsigns.admin.create")) {
            stringFormatter.displayErrorMessage(sender, Translatable.COMMAND_PERMISSION_DENIED);
            return false;
        }
        if (args.length < 3) {
            stringFormatter.displayErrorMessage(sender, Translatable.MISSING_CREATION_INFO);
            sender.sendMessage(usage);
            return true;
        }

        PermissionSign newSign = parseSign(sender, args);
        if (newSign == null) {
            sender.sendMessage(usage);
            return true;
        }
        SignCreationManager.addSignCreationRequest((Player) sender, newSign);

        stringFormatter.displaySuccessMessage(sender, Translatable.CREATION_REQUEST_CREATED);
        return true;
    }

    /**
     * Parses the permission sign given in user input
     *
     * @param sender    <p>The sender that initiated the command</p>
     * @param arguments <p>The given command arguments</p>
     * @return <p>The parsed sign, or null if it could not be parsed</p>
     */
    private PermissionSign parseSign(@NotNull CommandSender sender, @NotNull String[] arguments) {
        String name = arguments[0];
        String description = replaceUnderscoresWithSpaces(arguments[1]);
        String[] permissions = arguments[2].replaceAll("\\?", " ").split(",");
        StringFormatter stringFormatter = PermissionSigns.getInstance().getStringFormatter();

        for (String permission : permissions) {
            if (!permission.contains(":")) {
                continue;
            }

            String world = permission.split(":")[0];
            if (!world.equalsIgnoreCase("all") && !world.equalsIgnoreCase("any") &&
                    Bukkit.getWorld(world) == null) {

                stringFormatter.displayErrorMessage(sender, stringFormatter.replacePlaceholder(
                        Translatable.PERMISSION_WORLD_INVALID, "{world}", world));
                return null;
            }
        }

        double cost = 0;
        int duration = 0;
        if (arguments.length > 3) {
            try {
                cost = Double.parseDouble(arguments[3]);
            } catch (NumberFormatException exception) {
                stringFormatter.displayErrorMessage(sender, Translatable.COST_INVALID_NUMBER);
                return null;
            }
        }
        if (arguments.length > 4) {
            try {
                duration = Integer.parseInt(arguments[4]);
            } catch (NumberFormatException exception) {
                stringFormatter.displayErrorMessage(sender, Translatable.DURATION_INVALID_NUMBER);
                return null;
            }
        }

        return new PermissionSign(name, description, List.of(permissions), duration, cost);
    }

    /**
     * Replaces underscores with spaces in the given description
     *
     * <p>This basically replaces all underscores with spaces, but keeps escaped underscores (\_) as underscores.</p>
     *
     * @param description <p>The description to replace underscores in</p>
     * @return <p>The string with underscores replaced</p>
     */
    private @NotNull String replaceUnderscoresWithSpaces(@NotNull String description) {
        String temporaryString = "\\|ESCAPED-UNDERSCORE\\|";
        description = description.replaceAll("\\\\_", temporaryString);
        description = description.replaceAll("_", " ");
        description = description.replaceAll(temporaryString, "_");
        return description;
    }

}
