package net.knarcraft.permissionsigns.config;

import net.knarcraft.knarlib.formatting.Translator;
import org.bukkit.configuration.file.FileConfiguration;
import org.jetbrains.annotations.NotNull;

import java.io.File;

/**
 * PermissionSigns' configuration file
 */
public class Configuration {

    private final Translator translator;
    private final File dataFolder;
    private boolean perWorldPermissions;
    private boolean enableExtensiveSignProtection;
    private boolean enableIndirectSignProtection;
    private boolean removePermissionSignIfMissing;

    /**
     * Instantiates a new configuration
     *
     * @param fileConfiguration <p>The file configuration to load</p>
     * @param translator        <p>The translator to load translations for</p>
     * @param dataFolder        <p>This plugin's data folder</p>
     */
    public Configuration(@NotNull FileConfiguration fileConfiguration, @NotNull Translator translator,
                         @NotNull File dataFolder) {
        this.translator = translator;
        this.dataFolder = dataFolder;
        loadConfig(fileConfiguration);
    }

    /**
     * Loads the current configuration
     *
     * @param fileConfiguration <p>The file configuration to load</p>
     */
    public void loadConfig(@NotNull FileConfiguration fileConfiguration) {
        String language = fileConfiguration.getString("language", "en");
        perWorldPermissions = fileConfiguration.getBoolean("perWorldPermissions", false);
        enableExtensiveSignProtection = fileConfiguration.getBoolean("enableExtensiveSignProtection", false);
        enableIndirectSignProtection = fileConfiguration.getBoolean("enableIndirectSignProtection", true);
        removePermissionSignIfMissing = fileConfiguration.getBoolean("removePermissionSignIfMissing", true);
        translator.loadLanguages(dataFolder, "en", language);
    }

    /**
     * Checks whether permissions should always be set for the world the sign belongs to
     *
     * @return <p>Whether permissions should be set for the current world</p>
     */
    public boolean usePerWorldPermissions() {
        return perWorldPermissions;
    }

    /**
     * Gets whether permission signs on falling blocks should be protected
     *
     * @return <p>Whether permission signs on falling blocks should be protected</p>
     */
    public boolean extensiveSignProtectionEnabled() {
        return enableExtensiveSignProtection;
    }

    /**
     * Gets whether permission sign explosion and piston protection is enabled
     *
     * @return <p>Whether permission sign explosion and piston protection is enabled</p>
     */
    public boolean indirectProtectionEnabled() {
        return enableIndirectSignProtection;
    }

    /**
     * Gets whether to remove permissions signs if their physical signs are missing
     *
     * @return <p>Whether tor remove permission signs if their signs are missing</p>
     */
    public boolean removePermissionSignIfMissing() {
        return removePermissionSignIfMissing;
    }

}
